from aiohttp import ClientSession
from asyncio import create_task


class AsyncSocket:
	def __init__(self, headers, debug: bool = False):
		self.socket_url = "wss://gateway.discord.gg"
		self.debug=debug
		self.headers = headers

		self.task_resolve = None
		self.socket = None
		self.connection = None

	async def connect(self):
		try:
			if self.debug:
				print(f"[socket][start] Starting Socket")
			self.socket = ClientSession(base_url=self.socket_url)
			self.connection = await self.socket.ws_connect("/?encoding=json&v=9&compress=zlib-stream",headers=self.headers())
			self.task_resolve = create_task(self.resolve())
			if self.debug:
				print(f"[socket][start] Socket Started")
		except Exception as e:
			if self.debug:
				print(f"[socket][start] Error while starting Socket : {e}")


	async def disconnect(self):
		if self.debug:
			print(f"[socket][close] Closing Socket")
		try:
			if self.task_resolve: self.task_resolve.cancel()
			if self.connection:
				await self.connection.close()
				self.connection = None
			if self.socket:
				await self.socket.close()
				self.socket = None
			if self.debug:
				print(f"[socket][close] Socket closed")
		except Exception as e:
			if self.debug:
				print(f"[socket][close] Error while closing Socket : {e}")


	async def resolve(self):
		while True:
			msg = await self.connection.receive()
			print(msg)


from websocket import WebSocketApp, enableTrace
from threading import Thread


class Socket:
	def __init__(self, headers, debug: bool = False, sock_trace: bool = False):
		self.socket_url = "wss://gateway.discord.gg"
		self.socket = None
		self.debug=debug
		self.headers = headers
		enableTrace(sock_trace)



	def connect(self):
		try:
			if self.debug:
				print(f"[socket][start] Starting Socket")

			self.socket = WebSocketApp(
				f"{self.socket_url}/?encoding=json&v=9&compress=zlib-stream",
				header = self.headers(),
				on_message=self.resolve
			)
			self.socket_thread = Thread(target=self.socket.run_forever)
			self.socket_thread.start()

			if self.debug:
				print(f"[socket][start] Socket Started")
		except Exception as e:
			if self.debug:
				print(f"[socket][start] Error while starting Socket : {e}")


	def disconnect(self):
		if self.debug:
			print(f"[socket][close] Closing Socket")
		try:
			self.socket.close()
			self.socket_thread = None
			if self.debug:
				print(f"[socket][close] Socket closed")
		except Exception as e:
			if self.debug:
				print(f"[socket][close] Error while closing Socket : {e}")
		return


	def resolve(self, ws, data):
		try:
			#data = loads(data)
			print(data)
		except Exception as e:
			print(f"[socket][resolve] Error while resolve data : {e}")





class LoginInfo:
	def __init__(self, data: dict = {}):
		self.json = data
		user_settings = self.json.get("user_settings")
		
		self.token = self.json.get("token")
		self.userId = self.json.get("user_id")
		self.locale = user_settings.get("locale")
		self.theme = user_settings.get("theme")

class Message:
	def __init__(self, data: dict = {}):
		self.json = data
		self.messageId = self.json.get("id")
		self.messageType = self.json.get("type")
		self.content = self.json.get("content")
		self.channelId = self.json.get("channel_id")
		self.attachments = self.json.get("attachments", [])
		self.embeds = self.json.get("embeds", [])
		self.mentions = self.json.get("mentions", [])
		self.mention_roles = self.json.get("mention_roles", [])
		self.pinned = self.json.get("pinned")
		self.mention_everyone = self.json.get("mention_everyone")
		self.tts = self.json.get("tts")
		self.timestamp = self.json.get("timestamp")
		self.edited_timestamp = self.json.get("edited_timestamp")
		self.flags = self.json.get("flags")
		self.components = self.json.get("components", [])
		self.author = self.Author(self.json.get("author", {}))
		self.referenced_message = self.json.get("referenced_messages")

	class Author:
		def __init__(self, data: dict = {}):
			self.json = data
			self.userId = self.json.get("id")
			self.username = self.json.get("username")
			self.global_name = self.json.get("global_name")
			self.display_name = self.json.get("display_name")
			self.avatar = self.json.get("avatar")
			self.avatar_decoration = self.json.get("avatar_decoration")
			self.discriminator = self.json.get("discriminator")
			self.public_flags = self.json.get("public_flags")




class ChannelMessages:
	def __init__(self, data: list = []):
		self.json = data
		self.size = len(self.json)
		self.messages = list()
		for message in self.json:
			self.messages.append(Message(message))



class UserProfile:
	def __init__(self, data: dict = {}):
		self.json = data
		user = self.json.get("user", {})

		self.userId = user.get("id")
		self.username = user.get("username")
		self.global_name = user.get("global_name")
		self.display_name = user.get("display_name")
		self.avatar = user.get("avatar")
		self.avatar_decoration = user.get("avatar_decoration")
		self.discriminator = user.get("discriminator")
		self.public_flags = user.get("public_flags")
		self.flags = user.get("flags")
		self.banner = user.get("banner")
		self.banner_color = user.get("banner_color")
		self.accent_color = user.get("accent_color")
		self.bio = user.get("bio")
		self.premium_since = self.json.get("premium_since")
		self.premium_type = self.json.get("premium_type")
		self.premium_guild_since = self.json.get("premium_guild_since")
		self.profile_themes_experiment_bucket = self.json.get("profile_themes_experiment_bucket")
		self.mutual_friends_count = self.json.get("mutual_friends_count")
		self.mutual_guilds = self.json.get("mutual_guilds")
		self.connected_accounts = self.json.get("connected_accounts")


class Channels:
	def __init__(self, data: list = []):
		self.json = data
		self.size = len(self.json)
		self.channels = list()
		for channel in self.json:
			self.channels.append(self.ChannelInfo(channel))


	class ChannelInfo:
		def __init__(self, data: dict = {}):
			self.json=data
			self.channelId = self.json.get("id")
			self.channelType = self.json.get("type")
			self.last_message_id = self.json.get("last_message_id")
			self.flags = self.json.get("flags")
			self.name = self.json.get("name")
			self.icon = self.json.get("icon")
			self.owner_id = self.json.get("owner_id")
			self.channelMembers = list()
			for member in self.json.get("recipients", {}):
				self.channelMembers.append(UserProfile(member))
			self.members_size = len(self.channelMembers)

class Guilds:
	def __init__(self, data: list = []):
		self.json = data
		self.size = len(self.json)
		self.guilds = list()
		for guild in self.json:
			self.guilds.append(self.GuildInfo(guild))


	class GuildInfo:
		def __init__(self, data: dict = {}):
			self.json = data
			self.guildId = self.json.get("id")
			self.guildName = self.json.get("name")
			self.icon = self.json.get("icon")
			self.owner = self.json.get("owner")
			self.permissions = self.json.get("permissions")
			self.features = self.json.get("features", [])



from .utils.requester import Requester
from .utils import objects, exceptions

from .socket import Socket

from json import loads

class Client(Socket):
	def __init__(self, proxies: dict = None, socket_debug: bool = False, sock_trace: bool = False):
		self.req = Requester()
		self.proxies = proxies

		Socket.__init__(self, headers=self.req.headers, sock_trace=sock_trace, debug=socket_debug)


	def login(self, login: str, password: str, login_source: str = None, gift_code_sku_id: str = None):
		"""
		`Log in to your account`
		"""

		data = {
			"login": login,
			"password": password,
			"undelete": False,
			"captcha_key": None,
			"login_source": login_source,
			"gift_code_sku_id": gift_code_sku_id
		}

		resp = objects.LoginInfo(loads(self.req.make_request(method="POST", endpoint="/auth/login", body=data, proxies=self.proxies).text))
		self.req.token = resp.token
		#self.connect()
		return resp

	def logout(self):
		"""
		`Sign out of your account`
		"""

		data = {
			"provider": None,
			"voip_provider": None
		}

		resp = self.req.make_request(method="POST", endpoint="/auth/logout", body=data, allowed_code=204, proxies=self.proxies).status_code
		self.req.token = None
		return resp


	def get_channel_messages(self, channelId: int, size: int = 50):
		"""
		`Get a list of messages in a channel`
			**channelId - ID of the selected channel [TYPE: int]

			*size - Message list length [TYPE: int] [Default: 50]
		"""

		return objects.ChannelMessages(loads(self.req.make_request(method="GET", endpoint=f"/channels/{channelId}/messages?limit={size}", proxies=self.proxies).text))

	def get_user_info(self, userId: int, with_mutual_guilds: bool = True, with_mutual_friends_count: bool = True):
		"""
		`Get user information`
			**userId - ID of the selected user [TYPE: int]

			*with_mutual_guilds - Get shared servers [TYPE: bool] [Default: True]
			*with_mutual_friends_count - Get mutual friends [TYPE: bool] [Default: True]
		"""

		return objects.UserProfile(loads(self.req.make_request(method="GET", endpoint=f"/users/{userId}/profile?with_mutual_guilds={with_mutual_guilds}&with_mutual_friends_count={with_mutual_friends_count}", proxies=self.proxies).text))

	def get_my_channels(self):
		"""
		`Get a list of channels on an account`
		"""

		return objects.Channels(loads(self.req.make_request(method="GET", endpoint=f"/users/@me/channels", proxies=self.proxies).text))



	def get_my_guilds(self):
		"""
		`Get the servers you belong to`
		"""

		return objects.Guilds(loads(self.req.make_request(method="GET", endpoint=f"/users/@me/guilds", proxies=self.proxies).text))

	def send_message(self, channelId: int, message: str) -> dict:
		"""
		`Send a message to a channel`
			**channelId - Channel ID [TYPE: int]
			**message - Your message [TYPE: str]
		"""

		data = {"content": message}
		return objects.Message(loads(self.req.make_request(method="POST", endpoint=f"/channels/{channelId}/messages", body=data, proxies=self.proxies).text))


	def delete_message(self, channelId: int, messageId: int) -> dict:
		"""
		`Delete a message from a channel`
			**channelId - Channel ID [TYPE: int]
			**messageId - Message ID [TYPE: str]
		"""
		return self.req.make_request(method="DELETE", endpoint=f"/channels/{channelId}/messages/{messageId}", allowed_code=204, proxies=self.proxies).status_code


from .exceptions import *

from requests import Session
from aiohttp import ClientSession
from json import dumps

class Requester:
	def __init__(self):
		self.web_api = "https://discord.com/api/v9"
		self.session = Session()
		self.token = None

	def headers(self):
		headers = {
			"content-type": "application/json",
			"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/111.0"
		}
		if self.token:headers["authorization"] = self.token

		return headers


	def make_request(self, method: str, endpoint: str, body: dict = None, allowed_code: int = 200, proxies: dict = None):

		response = self.session.request(method, f"{self.web_api}{endpoint}", data=dumps(body) if body else None, headers=self.headers(), proxies=proxies)
		return checkException(response.text) if response.status_code != allowed_code else response


	async def make_async_request(self, method: str, endpoint: str, body: dict = None, allowed_code: int = 200):
		async with ClientSession() as asyncSession:
			response = await asyncSession.request(method, f"{self.web_api}{endpoint}", data=dumps(body) if body else None, headers=self.headers())
			return checkException(await response.text()) if response.status != allowed_code else response